@@ -27,18 +27,47 @@
  *
  */
 
-
 // TODO:
+//
+// * maybe important, requires deeper code changes:
+// - add config setting: use any connection/WLAN only
+//   - we don't use bearer information yet
+// - download another file with DHCP/WLAN client counters
+//   - better get them merged into the same file
+//   - download/processing function evolved from synchronous
+//     call, not suited for multiple URLs
+//   - use pango to draw numbers over icon
+//
+// * maybe good, requires reading more docs:
 // - sprinkle with type-checking macros and g_assert
+//   - how do they work anyway?
+// - check settings dialog for object/memory leaks
+//   - which parts are freed automatically, which are not?
+//   - is g_unref() the right tool?
+// - check return code from g_timeout_add_seconds/g_signal_connect
+//   - right now, we wouldn't know if start_timer()/install_handler() failed
+//   - what to do in case of error?
+// - does g_source_remove destroy the source or does it leak?
+//   - what data structures does a source use?
+//
+// * could be useful, requires more thinking:
+// - aggregate calls to force_redraw()
+//   - right now, we call force_redraw() on any possible change
+// - skip draw if not on active desktop
+//   - there's an event for that
+//   - don't skip update or banner
+//
+// * style issues, don't affect features:
 // - use more HildonCaption in settings dialog?
 //   - checkbox buttons look fine without Caption
 //   - PickerButton could use a larger font
-// - check settings dialog for object/memory leaks
-// - check return code from g_timeout_add_seconds/g_signal_connect
-// - does g_source_remove destroy the source or does it leak?
+//
+// * nice to have:
 // - draw nice icons
+//   - winni already did some work on new icons
 // - support multiple instances
-
+//   - to support future extensions
+//   - not much use now
 
 
 
@@ -69,9 +98,17 @@
 #include "ccchh-dooris-widget.h"
 
 
+/** This is like G_DEFINE_TYPE(), but it also 
+ * implements standard functions such as hd_plugin_module_load(), 
+ * which hildon-desktop expects to be able to call on this library.
+ */
+HD_DEFINE_PLUGIN_MODULE (CCCHHDoorIsDesktopWidget, ccchh_dooris_desktop_widget,
+  HD_TYPE_HOME_PLUGIN_ITEM)
 
 
-//#define DEBUG
+
+
+// #define DEBUG
 
 #ifdef DEBUG
 
@@ -90,33 +127,6 @@
 
 
 
-/** This is like G_DEFINE_TYPE(), but it also 
- * implements standard functions such as hd_plugin_module_load(), 
- * which hildon-desktop expects to be able to call on this library.
- */
-HD_DEFINE_PLUGIN_MODULE (CCCHHDoorIsDesktopWidget, ccchh_dooris_desktop_widget,
-  HD_TYPE_HOME_PLUGIN_ITEM)
-
-
-
-
-// stop update timer
-// returns TRUE if timer was stopped
-// returns FALSE if no timer was running
-static inline gboolean
-stop_update_timer(CCCHHDoorIsDesktopWidget *self)
-{
-	// stop timer if it was running
-	if (self->timeout_handler) {
-		g_debug(WIDGET_NAME": stopping update timer");
-		g_source_remove (self->timeout_handler);
-		self->timeout_handler = 0;
-		return TRUE;
-	}
-	return FALSE;
-}
-
-
 // start update timer
 // returns TRUE if timer was started
 // returns FALSE if timer was already running or auto_update disabled
@@ -143,6 +153,23 @@ start_update_timer(CCCHHDoorIsDesktopWidget *self)
 }
 
 
+// stop update timer
+// returns TRUE if timer was stopped
+// returns FALSE if no timer was running
+static inline gboolean
+stop_update_timer(CCCHHDoorIsDesktopWidget *self)
+{
+	// stop timer if it was running
+	if (self->timeout_handler) {
+		g_debug(WIDGET_NAME": stopping update timer");
+		g_source_remove (self->timeout_handler);
+		self->timeout_handler = 0;
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 // install libconic connection event handler
 // returns TRUE if handler was installed
 // returns FALSE if handler was already set
@@ -187,6 +214,7 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 
 
 // load settings from gconf
+// TODO: use gconf_client_get_without_default()?
 static void
 load_settings(CCCHHDoorIsDesktopWidget *self)
 {
@@ -460,8 +488,7 @@ show_status_banner (CCCHHDoorIsDesktopWidget *self)
 
 	// display banner on screen
 	hildon_banner_show_information_with_markup (GTK_WIDGET (self),
-		NULL, /* icon_name - ignored */
-		text);
+		NULL, text);
 
 	g_free (text);
 	g_free (ts_text);
@@ -491,15 +518,6 @@ force_redraw (CCCHHDoorIsDesktopWidget *self)
 	gdk_region_destroy (region);
 }
 
-static void
-force_redraw_window (GtkWindow *window)
-{
-	g_debug(WIDGET_NAME": force_redraw_window()");
-
-	// process expose event
-	gdk_window_process_updates (window, TRUE);
-}
-
 
 
 
@@ -829,9 +847,9 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 	// connection event starts it again
 	if (!self->connected) {
 		g_debug(WIDGET_NAME": not connected, stopping update timer");
-// TODO: is this safe? how about remove_source from within the event?
-self->timeout_handler = 0;
-force_redraw(self);
+		// TODO: is this safe? how about remove_source from within the event?
+		self->timeout_handler = 0;
+		force_redraw(self);
 		return FALSE;
 	}
 
@@ -1059,118 +1077,113 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_on_settings_dialog_response()");
 
 	// only use new settings if user pressed the OK button
-	if (response_id == GTK_RESPONSE_ACCEPT) {
-		g_debug(WIDGET_NAME": response accept");
+	switch (response_id) {
+		case GTK_RESPONSE_NO:
+			g_debug(WIDGET_NAME": restoring default configuration");
 
-		// status URL
-		if (self->update_url) {
-			free(self->update_url);
-			self->update_url = NULL;
-		}
+			gtk_entry_set_text(GTK_ENTRY(self->url_entry), DEFAULT_URL);
 
-		// try to build a valid URI from user input
-		self->update_url = gnome_vfs_make_uri_from_input(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
-		g_debug(WIDGET_NAME": new URL: %s\n", self->update_url);
+			hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->notify_button), TRUE);
+			hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connforce_button), TRUE);
+			hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connection_button), TRUE);
 
+			// TODO: avoid index constant
+			hildon_touch_selector_set_active(
+				hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(self->update_picker)),
+				0, 4);
 
-		// update interval
-		const gchar *choice = hildon_button_get_value(HILDON_BUTTON (self->update_picker));
-		if (strcmp(choice, _("disabled"))) {
-			// automatic updates enabled
-			self->auto_update = TRUE;
-			self->update_interval = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
-			g_debug(WIDGET_NAME": new update interval: %ld\n", (long)self->update_interval);
+			hildon_button_set_value(HILDON_BUTTON (self->update_picker), "23");
 
-			// reset timer
-			stop_update_timer(self);
-			start_update_timer(self);
-		} else {
-			// automatic updates disabled
-			self->auto_update = FALSE;
+			// keep dialog window open
 
-			g_debug(WIDGET_NAME": automatic updates disabled");
+			break;
 
-			// if a timer was running, stop it
-			stop_update_timer(self);
-		}
+		case GTK_RESPONSE_ACCEPT:
+			g_debug(WIDGET_NAME": response accept");
 
-		// banner notifications
-		self->notify_enabled = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->notify_button));
-		if (self->notify_enabled) {
-			g_debug(WIDGET_NAME": notification enabled");
-		} else {
-			g_debug(WIDGET_NAME": notification disabled");
-		}
-
-		// monitor internet connection state, only update if online
-		self->use_connection = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connection_button));
-
-		if (self->use_connection) {
-			install_connection_handler(self);
-		} else {
-			remove_connection_handler(self);
-
-			// force connection state to online
-			// redraw if necessary to remove disconnection overlay
-			if (!self->connected) {
-				self->connected = TRUE;
-				force_redraw(self);
+			// status URL
+			if (self->update_url) {
+				free(self->update_url);
+				self->update_url = NULL;
 			}
 
-			// start timer (if auto_update enabled, too)
-			start_update_timer(self);
-		}
+			// try to build a valid URI from user input
+			self->update_url = gnome_vfs_make_uri_from_input(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
+			g_debug(WIDGET_NAME": new URL: %s\n", self->update_url);
 
-		// button forces internet connection
-		self->force_connection = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connforce_button));
-		if (self->force_connection) {
-			g_debug(WIDGET_NAME": button forces connection");
-		} else {
-			g_debug(WIDGET_NAME": button does not force connection");
-		}
+			// update interval
+			const gchar *choice = hildon_button_get_value(HILDON_BUTTON (self->update_picker));
+			if (strcmp(choice, _("disabled"))) {
+				// automatic updates enabled
+				self->auto_update = TRUE;
+				self->update_interval = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
+				g_debug(WIDGET_NAME": new update interval: %ld\n", (long)self->update_interval);
 
-		save_settings(self);
+				// reset timer
+				stop_update_timer(self);
+				start_update_timer(self);
+			} else {
+				// automatic updates disabled
+				self->auto_update = FALSE;
 
-		gtk_widget_destroy (self->settings_window);
-		// TODO: does this leak any object references such as GtkSizeGroup
-		// or HildonCaption?
-		// isn't there a better way to this? array+memset?
-		self->settings_window = NULL;
-		self->url_entry = NULL;
-		self->update_picker = NULL;
-		self->notify_button = NULL;
-		self->connection_button = NULL;
-		self->connforce_button = NULL;
+				g_debug(WIDGET_NAME": automatic updates disabled");
 
-	} else if (response_id == GTK_RESPONSE_NO) {
-		g_debug(WIDGET_NAME": restoring default configuration");
+				// if a timer was running, stop it
+				stop_update_timer(self);
+			}
 
-		gtk_entry_set_text(GTK_ENTRY(self->url_entry), "http://dooris.koalo.de/door.txt");
-//		gtk_entry_set_text(GTK_ENTRY(self->url_entry), "http://formularfetischisten.de/~packbart/temp/door.txt");
+			// banner notifications
+			self->notify_enabled = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->notify_button));
+			if (self->notify_enabled) {
+				g_debug(WIDGET_NAME": notification enabled");
+			} else {
+				g_debug(WIDGET_NAME": notification disabled");
+			}
 
-		hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->notify_button), TRUE);
-		hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connforce_button), TRUE);
-		hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connection_button), TRUE);
+			// monitor internet connection state, only update if online
+			self->use_connection = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connection_button));
 
-		hildon_touch_selector_set_active(
-			hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(self->update_picker)),
-			0, 4);
+			if (self->use_connection) {
+				install_connection_handler(self);
+			} else {
+				remove_connection_handler(self);
 
-		hildon_button_set_value(HILDON_BUTTON (self->update_picker), "23");
+				// force connection state to online
+				// redraw if necessary to remove disconnection overlay
+				if (!self->connected) {
+					self->connected = TRUE;
+					force_redraw(self);
+				}
 
-	} else {
-		g_debug(WIDGET_NAME": not saving new settings, response ID was %i", response_id);
+				// start timer (if auto_update enabled, too)
+				start_update_timer(self);
+			}
 
-		gtk_widget_destroy (self->settings_window);
-		// TODO: does this leak any object references such as GtkSizeGroup
-		// or HildonCaption?
-		// isn't there a better way to this? array+memset?
-		self->settings_window = NULL;
-		self->url_entry = NULL;
-		self->update_picker = NULL;
-		self->notify_button = NULL;
-		self->connection_button = NULL;
-		self->connforce_button = NULL;
+			// button forces internet connection
+			self->force_connection = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connforce_button));
+			if (self->force_connection) {
+				g_debug(WIDGET_NAME": button forces connection");
+			} else {
+				g_debug(WIDGET_NAME": button does not force connection");
+			}
+
+			save_settings(self);
+
+			// fall through to dispose dialog window
+
+		default:
+			g_debug(WIDGET_NAME": response ID was %i", response_id);
+
+			gtk_widget_destroy (self->settings_window);
+			// TODO: does this leak any object references such as GtkSizeGroup
+			// or HildonCaption?
+			// isn't there a better way to this? array+memset?
+			self->settings_window = NULL;
+			self->url_entry = NULL;
+			self->update_picker = NULL;
+			self->notify_button = NULL;
+			self->connection_button = NULL;
+			self->connforce_button = NULL;
 	}
 }
 
@@ -1191,6 +1204,7 @@ void update_picker_changed_value_cb (HildonPickerButton *picker, gpointer data)
 	}
 }
 
+
 // use_connection button callback - set force_connection (in-)sensitive
 void connection_button_toggled_cb (HildonCheckButton *button, gpointer data)
 {
@@ -1420,9 +1434,6 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 		cairo_surface_destroy(self->image_current);
 	}
 
-	// don'shut down gnome vfs library here
-	// other parts of the desktop might still use it
- 
 	if (self->update_url) {
 		free(self->update_url);
 		self->update_url = NULL;
@@ -1499,10 +1510,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	// default settings
 	self->notify_enabled = TRUE;
 	self->update_interval = 23;
-	// TODO: this should be moved to turing.hamburg.ccc.de
-	self->update_url = gnome_vfs_make_uri_from_input("http://dooris.koalo.de/door.txt");
-//	self->update_url = gnome_vfs_make_uri_from_input("http://formularfetischisten.de/~packbart/lustitsch/whale.mov");
-//	self->update_url = gnome_vfs_make_uri_from_input("http://formularfetischisten.de/~packbart/temp/door.txt");
+	self->update_url = gnome_vfs_make_uri_from_input(DEFAULT_URL);
 	self->use_connection = TRUE;
 	self->force_connection = TRUE;
 	self->auto_update = TRUE;
