@@ -10,15 +10,21 @@
 // gconf base path
 #define GCONF_DIR "/apps/maemo/ccchh-dooris-widget"
 
+// default configuration
+// TODO: this should be moved to turing.hamburg.ccc.de
+#define DEFAULT_URL "http://dooris.koalo.de/door.txt"
+//#define DEFAULT_URL "http://formularfetischisten.de/~packbart/temp/door.txt"
+//#define DEFAULT_URL "http://formularfetischisten.de/~packbart/lustitsch/whale.mov"
+
 // maximum report age in seconds
 #define REPORT_TIMEOUT 7200
 
-// async I/O timeouts
+// async I/O timeouts in seconds
 #define TIMEOUT_CONNECT 5
 #define TIMEOUT_READ	30
 #define TIMEOUT_CLOSE	5
 
-// max download size
+// max download size in bytes
 #define MAX_HTTP_SIZE 1024
 
 // minimum/maximum selectable update interval
@@ -32,7 +38,7 @@
 #define IMAGE_DOORIS_FNORD			DATADIR"/dooris_fnord.png"
 #define IMAGE_OVERLAY_DISCONNECTED	DATADIR"/dooris_notavail.png"
 
-// widget size
+// widget size in pixels
 #define IMAGE_WIDGET_WIDTH	102
 #define IMAGE_WIDGET_HEIGHT	72
 
